import 'dart:async';
import 'dart:io';

import 'package:async/async.dart';
import 'package:gaza_machine/src/generated/grpc/machine/machine.pbgrpc.dart';
import 'package:gaza_machine/src/generated/grpc/machine/machine.pbgrpc.service.dart';
import 'package:path/path.dart';

extension on String {
  Directory get dir => Directory(this);
  File get file => File(this);
}

class ShellService extends BetterShellServiceBase {
  String pMachineDir;
  ShellService(this.pMachineDir);
  final _processes = <String, Process>{};

  static String _newId() => DateTime.now().millisecondsSinceEpoch.toString();

  String get pProcessesDir => join(pMachineDir, "processes");
  String pProcessDir(String id) => join(pProcessesDir, id);
  String pLogsDir(String id) => join(pProcessDir(id), "logs");
  String pStdoutLog(String id) => join(pLogsDir(id), "stdout.txt");
  String pStderrLog(String id) => join(pLogsDir(id), "stderr.txt");

  @override
  Stream<CommandLog> getCommandLogs(CommandId request) async* {
    int position = 0;
    while ((await isRunning(request)).value == true) {}
  }

  @override
  FutureOr<Bool> isRunning(CommandId request) =>
      Bool(value: _processes.containsKey(request.value));

  @override
  FutureOr<Bool> killCommand(CommandId request) {
    throw UnimplementedError();
  }

  @override
  FutureOr<CommandId> startCommand(StartCommandRequest request) async {
    final id = _newId();
    final process = await Process.start(request.executable, request.arguments);

    final processDir = pProcessDir(id).dir;
    if (!processDir.existsSync()) processDir.createSync(recursive: true);
    _processes[id] = process;

    final fStdoutLog = pStdoutLog(id).file..createSync(recursive: true);
    final fStderrLog = pStdoutLog(id).file..createSync(recursive: true);

    final stdoutStream = process.stdout
        .map((bytes) => CommandLog(type: CommandLogType.stdout, bytes: bytes));
    final stderrStream = process.stderr
        .map((bytes) => CommandLog(type: CommandLogType.stderr, bytes: bytes));

    final sub =
        StreamGroup.merge([stdoutStream, stderrStream]).listen((log) async {
      final File file;
      if (log.type == CommandLogType.stdout) {
        file = fStdoutLog;
      } else {
        file = fStderrLog;
      }
      file.writeAsBytesSync(log.bytes, mode: FileMode.append);
    });
    sub.asFuture().then((_) {
      _processes.remove(id);
    });

    return CommandId(value: id);
  }
}
