// ignore_for_file: implementation_imports

import 'dart:async';
import 'dart:io' as io;

import 'package:gaza_machine/src/generated/grpc/machine/machine.pbgrpc.dart';
import 'package:gaza_machine/src/generated/grpc/machine/machine.pbgrpc.service.dart';
import 'package:path/path.dart';
import 'package:system_info2/system_info2.dart';

class SystemService extends BetterSystemServiceBase {
  String pMachineDir;
  SystemService(this.pMachineDir);

  @override
  Stream<EnviromentVariable> getEnviromentVariables(Empty request) async* {
    for (final variable in io.Platform.environment.entries) {
      yield EnviromentVariable(key: variable.key, value: variable.value);
    }
  }

  @override
  FutureOr<OSResponse> getOS(Empty request) {
    final OS os;
    if (io.Platform.isLinux) {
      os = OS.linux;
    } else if (io.Platform.isWindows) {
      os = OS.windows;
    } else if (io.Platform.isMacOS) {
      os = OS.macos;
    } else {
      os = OS.unknownOS;
    }
    return OSResponse(value: os);
  }

  @override
  Stream<FileSystemEntity> listDir(ListDirRequest request) async* {
    final dir = io.Directory(request.path);

    await for (final entity in dir.list()) {
      final statt = entity.statSync();

      final FileSystemEntityType type;
      if (statt.type == io.FileSystemEntityType.directory) {
        type = FileSystemEntityType.dir;
      } else if (statt.type == io.FileSystemEntityType.file) {
        type = FileSystemEntityType.file;
      } else {
        type = FileSystemEntityType.unknownType;
      }
      yield FileSystemEntity(type: type, name: dirname(entity.path));
    }
  }

  Future<Map<String, String>> get enviromentVariables async {
    final map = <String, String>{};
    await for (final v in getEnviromentVariables(Empty())) {
      map[v.key] = v.value;
    }
    return map;
  }

  static final _architectureMap = {
    ProcessorArchitecture.arm: Arch.arm,
    ProcessorArchitecture.unknown: Arch.unknownArch,
    ProcessorArchitecture.x86_64: Arch.x86_64,
    ProcessorArchitecture.aarch64: Arch.aarch64,
    ProcessorArchitecture.ia64: Arch.ia64,
    ProcessorArchitecture.mips: Arch.mips,
    ProcessorArchitecture.x86: Arch.x86,
  };
  @override
  FutureOr<ArchResponse> getArch(Empty request) async {
    final sysArch = SysInfo.cores.first.architecture;
    return ArchResponse(arch: _architectureMap[sysArch]);
  }
}
